#ifndef OMPL_UTILITIES
#define OMPL_UTILITIES

#include <iostream>
#include <ros/ros.h>
#include <fcl/collision.h>

template <typename S>
struct CollisionData
{
  CollisionData()
  {
    done = false;
  }

  /// @brief Collision request
  fcl::CollisionRequest<S> request;

  /// @brief Collision result
  fcl::CollisionResult<S> result;

  /// @brief Whether the collision iteration can stop
  bool done;
};

template <typename S>
bool defaultCollisionFunction(fcl::CollisionObject<S>* o1, fcl::CollisionObject<S>* o2, void* cdata_)
{
  auto* cdata = static_cast<CollisionData<S>*>(cdata_);
  const auto& request = cdata->request;
  auto& result = cdata->result;

  if(cdata->done) return true;

  collide(o1, o2, request, result);

  if(!request.enable_cost && (result.isCollision()) && (result.numContacts() >= request.num_max_contacts))
    cdata->done = true;

  return cdata->done;
}

bool legalNumber(char *str)
{
    while (*str != 0)
    {
        if (!isdigit(*str++))
        {
            return false;
        }
    }
    return true;
}

class DummyState{
    public:
    DummyState(double x, double y, double z, double yaw) :
    x(x), y(y), z(z), yaw(yaw)
    {
    }

    friend std::ostream& operator<<(std::ostream& os, const DummyState& d)
    {
	os << d.x << " ; " << d.y << " ; " << d.z << " ; " << d.yaw;
	return os;
    }

    double x;
    double y;
    double z;
    double yaw;
};

class NoFlyZone{
	public:
	NoFlyZone(double x, double y, double heigth, double radius):
    x(x), y(y), heigth(heigth), radius(radius)
	{
	}

    double x;
    double y;
    double heigth;
	double radius;
};

#endif
